@@ -95,10 +95,13 @@ static const char *FILE_GIF  =  "/tmp/lept/format/file.gif";
 static const char *FILE_WEBP =  "/tmp/lept/format/file.webp";
 static const char *FILE_JP2K =  "/tmp/lept/format/file.jp2";
 
+    /* There are two jp2 formats, and two codecs associated with them:
+     *    OPJ_CODEC_J2K (L_J2K_CODEC) is associated with JP2K_CODESTREAM
+     *    OPJ_CODEC_JP2 (L_JP2_CODEC) is associated with JP2K_IMAGE_DATA    */
 static const unsigned char JP2K_CODESTREAM[4] = { 0xff, 0x4f, 0xff, 0x51 };
-static const unsigned char JP2K_IMAGE_DATA[12] = { 0x00, 0x00, 0x00, 0x0C,
-                                                   0x6A, 0x50, 0x20, 0x20,
-                                                   0x0D, 0x0A, 0x87, 0x0A };
+static const unsigned char JP2K_IMAGE_DATA[12] = { 0x00, 0x00, 0x00, 0x0c,
+                                                   0x6a, 0x50, 0x20, 0x20,
+                                                   0x0d, 0x0a, 0x87, 0x0a };
 
 
 /*---------------------------------------------------------------------*
@@ -127,13 +130,11 @@ pixaReadFiles(const char  *dirname,
 PIXA    *pixa;
 SARRAY  *sa;
 
-    PROCNAME("pixaReadFiles");
-
     if (!dirname)
-        return (PIXA *)ERROR_PTR("dirname not defined", procName, NULL);
+        return (PIXA *)ERROR_PTR("dirname not defined", __func__, NULL);
 
     if ((sa = getSortedPathnamesInDirectory(dirname, substr, 0, 0)) == NULL)
-        return (PIXA *)ERROR_PTR("sa not made", procName, NULL);
+        return (PIXA *)ERROR_PTR("sa not made", __func__, NULL);
 
     pixa = pixaReadFilesSA(sa);
     sarrayDestroy(&sa);
@@ -155,17 +156,15 @@ l_int32  i, n;
 PIX     *pix;
 PIXA    *pixa;
 
-    PROCNAME("pixaReadFilesSA");
-
     if (!sa)
-        return (PIXA *)ERROR_PTR("sa not defined", procName, NULL);
+        return (PIXA *)ERROR_PTR("sa not defined", __func__, NULL);
 
     n = sarrayGetCount(sa);
     pixa = pixaCreate(n);
     for (i = 0; i < n; i++) {
         str = sarrayGetString(sa, i, L_NOCOPY);
         if ((pix = pixRead(str)) == NULL) {
-            L_WARNING("pix not read from file %s\n", procName, str);
+            L_WARNING("pix not read from file %s\n", __func__, str);
             continue;
         }
         pixaAddPix(pixa, pix, L_INSERT);
@@ -192,19 +191,16 @@ pixRead(const char  *filename)
 FILE  *fp;
 PIX   *pix;
 
-    PROCNAME("pixRead");
-
     if (!filename)
-        return (PIX *)ERROR_PTR("filename not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("filename not defined", __func__, NULL);
 
-    if ((fp = fopenReadStream(filename)) == NULL) {
-        L_ERROR("image file not found: %s\n", procName, filename);
-        return NULL;
-    }
+    if ((fp = fopenReadStream(filename)) == NULL)
+        return (PIX*)ERROR_PTR_1("image file not found",
+                                 filename, __func__, NULL);
     pix = pixReadStream(fp, 0);
     fclose(fp);
     if (!pix)
-        return (PIX *)ERROR_PTR("pix not read", procName, NULL);
+        return (PIX *)ERROR_PTR_1("pix not read", filename, __func__, NULL);
     return pix;
 }
 
@@ -230,18 +226,18 @@ pixReadWithHint(const char  *filename,
 FILE  *fp;
 PIX   *pix;
 
-    PROCNAME("pixReadWithHint");
-
     if (!filename)
-        return (PIX *)ERROR_PTR("filename not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("filename not defined", __func__, NULL);
 
     if ((fp = fopenReadStream(filename)) == NULL)
-        return (PIX *)ERROR_PTR("image file not found", procName, NULL);
+        return (PIX *)ERROR_PTR_1("image file not found",
+                                  filename, __func__, NULL);
     pix = pixReadStream(fp, hint);
     fclose(fp);
 
     if (!pix)
-        return (PIX *)ERROR_PTR("image not returned", procName, NULL);
+        return (PIX *)ERROR_PTR_1("image not returned",
+                                  filename, __func__, NULL);
     return pix;
 }
 
@@ -282,20 +278,18 @@ char    *fname;
 l_int32  n;
 PIX     *pix;
 
-    PROCNAME("pixReadIndexed");
-
     if (!sa)
-        return (PIX *)ERROR_PTR("sa not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("sa not defined", __func__, NULL);
     n = sarrayGetCount(sa);
     if (index < 0 || index >= n)
-        return (PIX *)ERROR_PTR("index out of bounds", procName, NULL);
+        return (PIX *)ERROR_PTR("index out of bounds", __func__, NULL);
 
     fname = sarrayGetString(sa, index, L_NOCOPY);
     if (fname[0] == '\0')
         return NULL;
 
     if ((pix = pixRead(fname)) == NULL) {
-        L_ERROR("pix not read from file %s\n", procName, fname);
+        L_ERROR("pix not read from file %s\n", __func__, fname);
         return NULL;
     }
 
@@ -324,10 +318,8 @@ l_uint8  *comment;
 PIX      *pix;
 PIXCMAP  *cmap;
 
-    PROCNAME("pixReadStream");
-
     if (!fp)
-        return (PIX *)ERROR_PTR("stream not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("stream not defined", __func__, NULL);
     pix = NULL;
 
     findFileFormatStream(fp, &format);
@@ -335,12 +327,12 @@ PIXCMAP  *cmap;
     {
     case IFF_BMP:
         if ((pix = pixReadStreamBmp(fp)) == NULL )
-            return (PIX *)ERROR_PTR( "bmp: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR( "bmp: no pix returned", __func__, NULL);
         break;
 
     case IFF_JFIF_JPEG:
         if ((pix = pixReadStreamJpeg(fp, 0, 1, NULL, hint)) == NULL)
-            return (PIX *)ERROR_PTR( "jpeg: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR( "jpeg: no pix returned", __func__, NULL);
         ret = fgetJpegComment(fp, &comment);
         if (!ret && comment)
             pixSetText(pix, (char *)comment);
@@ -349,7 +341,7 @@ PIXCMAP  *cmap;
 
     case IFF_PNG:
         if ((pix = pixReadStreamPng(fp)) == NULL)
-            return (PIX *)ERROR_PTR("png: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("png: no pix returned", __func__, NULL);
         break;
 
     case IFF_TIFF:
@@ -361,55 +353,55 @@ PIXCMAP  *cmap;
     case IFF_TIFF_ZIP:
     case IFF_TIFF_JPEG:
         if ((pix = pixReadStreamTiff(fp, 0)) == NULL)  /* page 0 by default */
-            return (PIX *)ERROR_PTR("tiff: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("tiff: no pix returned", __func__, NULL);
         break;
 
     case IFF_PNM:
         if ((pix = pixReadStreamPnm(fp)) == NULL)
-            return (PIX *)ERROR_PTR("pnm: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("pnm: no pix returned", __func__, NULL);
         break;
 
     case IFF_GIF:
         if ((pix = pixReadStreamGif(fp)) == NULL)
-            return (PIX *)ERROR_PTR("gif: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("gif: no pix returned", __func__, NULL);
         break;
 
     case IFF_JP2:
         if ((pix = pixReadStreamJp2k(fp, 1, NULL, 0, 0)) == NULL)
-            return (PIX *)ERROR_PTR("jp2: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("jp2: no pix returned", __func__, NULL);
         break;
 
     case IFF_WEBP:
         if ((pix = pixReadStreamWebP(fp)) == NULL)
-            return (PIX *)ERROR_PTR("webp: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("webp: no pix returned", __func__, NULL);
         break;
 
     case IFF_PS:
-        L_ERROR("PostScript reading is not supported\n", procName);
+        L_ERROR("PostScript reading is not supported\n", __func__);
         return NULL;
 
     case IFF_LPDF:
-        L_ERROR("Pdf reading is not supported\n", procName);
+        L_ERROR("Pdf reading is not supported\n", __func__);
         return NULL;
 
     case IFF_SPIX:
         if ((pix = pixReadStreamSpix(fp)) == NULL)
-            return (PIX *)ERROR_PTR("spix: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("spix: no pix returned", __func__, NULL);
         break;
 
     case IFF_UNKNOWN:
         return (PIX *)ERROR_PTR( "Unknown format: no pix returned",
-                procName, NULL);
+                __func__, NULL);
         break;
     }
 
     if (pix) {
         pixSetInputFormat(pix, format);
         if ((cmap = pixGetColormap(pix))) {
-            pixcmapIsValid(cmap, &valid);
+            pixcmapIsValid(cmap, pix, &valid);
             if (!valid) {
                 pixDestroy(&pix);
-                return (PIX *)ERROR_PTR("invalid colormap", procName, NULL);
+                return (PIX *)ERROR_PTR("invalid colormap", __func__, NULL);
             }
         }
     }
@@ -453,8 +445,6 @@ l_int32  type;  /* ignored */
 FILE    *fp;
 PIX     *pix;
 
-    PROCNAME("pixReadHeader");
-
     if (pw) *pw = 0;
     if (ph) *ph = 0;
     if (pbps) *pbps = 0;
@@ -463,10 +453,10 @@ PIX     *pix;
     if (pformat) *pformat = 0;
     iscmap = 0;  /* init to false */
     if (!filename)
-        return ERROR_INT("filename not defined", procName, 1);
+        return ERROR_INT("filename not defined", __func__, 1);
 
     if ((fp = fopenReadStream(filename)) == NULL)
-        return ERROR_INT("image file not found", procName, 1);
+        return ERROR_INT_1("image file not found", filename, __func__, 1);
     findFileFormatStream(fp, &format);
     fclose(fp);
 
@@ -474,26 +464,27 @@ PIX     *pix;
     {
     case IFF_BMP:  /* cheating: reading the entire file */
         if ((pix = pixRead(filename)) == NULL)
-            return ERROR_INT( "bmp: pix not read", procName, 1);
+            return ERROR_INT_1( "bmp: pix not read", filename, __func__, 1);
         pixGetDimensions(pix, &w, &h, &d);
-        if (pixGetColormap(pix))
-            iscmap = 1;
-        pixDestroy(&pix);
         bps = (d == 32) ? 8 : d;
-        spp = (d == 32) ? 3 : 1;
+        spp = pixGetSpp(pix);
+        iscmap = (pixGetColormap(pix)) ? 1 : 0;
+        pixDestroy(&pix);
         break;
 
     case IFF_JFIF_JPEG:
         ret = readHeaderJpeg(filename, &w, &h, &spp, NULL, NULL);
         bps = 8;
         if (ret)
-            return ERROR_INT( "jpeg: no header info returned", procName, 1);
+            return ERROR_INT_1("jpeg: no header info returned",
+                               filename, __func__, 1);
         break;
 
     case IFF_PNG:
         ret = readHeaderPng(filename, &w, &h, &bps, &spp, &iscmap);
         if (ret)
-            return ERROR_INT( "png: no header info returned", procName, 1);
+            return ERROR_INT_1("png: no header info returned",
+                               filename, __func__, 1);
         break;
 
     case IFF_TIFF:
@@ -508,18 +499,20 @@ PIX     *pix;
         ret = readHeaderTiff(filename, 0, &w, &h, &bps, &spp, NULL, &iscmap,
                              &format);
         if (ret)
-            return ERROR_INT( "tiff: no header info returned", procName, 1);
+            return ERROR_INT_1("tiff: no header info returned",
+                               filename, __func__, 1);
         break;
 
     case IFF_PNM:
         ret = readHeaderPnm(filename, &w, &h, &d, &type, &bps, &spp);
         if (ret)
-            return ERROR_INT( "pnm: no header info returned", procName, 1);
+            return ERROR_INT_1("pnm: no header info returned",
+                               filename, __func__, 1);
         break;
 
     case IFF_GIF:  /* cheating: reading the entire file */
         if ((pix = pixRead(filename)) == NULL)
-            return ERROR_INT( "gif: pix not read", procName, 1);
+            return ERROR_INT_1( "gif: pix not read", filename, __func__, 1);
         pixGetDimensions(pix, &w, &h, &d);
         pixDestroy(&pix);
         iscmap = 1;  /* always colormapped; max 256 colors */
@@ -528,33 +521,33 @@ PIX     *pix;
         break;
 
     case IFF_JP2:
-        ret = readHeaderJp2k(filename, &w, &h, &bps, &spp);
+        ret = readHeaderJp2k(filename, &w, &h, &bps, &spp, NULL);
         break;
 
     case IFF_WEBP:
         if (readHeaderWebP(filename, &w, &h, &spp))
-            return ERROR_INT( "webp: no header info returned", procName, 1);
+            return ERROR_INT_1("webp: no header info returned",
+                               filename, __func__, 1);
         bps = 8;
         break;
 
     case IFF_PS:
         if (pformat) *pformat = format;
-        return ERROR_INT("PostScript reading is not supported\n", procName, 1);
+        return ERROR_INT("PostScript reading is not supported\n", __func__, 1);
 
     case IFF_LPDF:
         if (pformat) *pformat = format;
-        return ERROR_INT("Pdf reading is not supported\n", procName, 1);
+        return ERROR_INT("Pdf reading is not supported\n", __func__, 1);
 
     case IFF_SPIX:
         ret = readHeaderSpix(filename, &w, &h, &bps, &spp, &iscmap);
         if (ret)
-            return ERROR_INT( "spix: no header info returned", procName, 1);
+            return ERROR_INT_1("spix: no header info returned",
+                               filename, __func__, 1);
         break;
 
     case IFF_UNKNOWN:
-        L_ERROR("unknown format in file %s\n", procName, filename);
-        return 1;
-        break;
+        return ERROR_INT_1("unknown format in file", filename, __func__, 1);
     }
 
     if (pw) *pw = w;
@@ -584,16 +577,14 @@ findFileFormat(const char  *filename,
 l_int32  ret;
 FILE    *fp;
 
-    PROCNAME("findFileFormat");
-
     if (!pformat)
-        return ERROR_INT("&format not defined", procName, 1);
+        return ERROR_INT("&format not defined", __func__, 1);
     *pformat = IFF_UNKNOWN;
     if (!filename)
-        return ERROR_INT("filename not defined", procName, 1);
+        return ERROR_INT("filename not defined", __func__, 1);
 
     if ((fp = fopenReadStream(filename)) == NULL)
-        return ERROR_INT("image file not found", procName, 1);
+        return ERROR_INT_1("image file not found", filename, __func__, 1);
     ret = findFileFormatStream(fp, pformat);
     fclose(fp);
     return ret;
@@ -616,23 +607,22 @@ l_ok
 findFileFormatStream(FILE     *fp,
                      l_int32  *pformat)
 {
-l_uint8  firstbytes[12];
+l_uint8  firstbytes[13];
 l_int32  format;
 
-    PROCNAME("findFileFormatStream");
-
     if (!pformat)
-        return ERROR_INT("&format not defined", procName, 1);
+        return ERROR_INT("&format not defined", __func__, 1);
     *pformat = IFF_UNKNOWN;
     if (!fp)
-        return ERROR_INT("stream not defined", procName, 1);
+        return ERROR_INT("stream not defined", __func__, 1);
 
     rewind(fp);
     if (fnbytesInFile(fp) < 12)
-        return ERROR_INT("truncated file", procName, 1);
+        return ERROR_INT("truncated file", __func__, 1);
 
     if (fread(&firstbytes, 1, 12, fp) != 12)
-        return ERROR_INT("failed to read first 12 bytes of file", procName, 1);
+        return ERROR_INT("failed to read first 12 bytes of file", __func__, 1);
+    firstbytes[12] = 0;
     rewind(fp);
 
     findFileFormatBuffer(firstbytes, &format);
@@ -669,13 +659,11 @@ findFileFormatBuffer(const l_uint8  *buf,
 {
 l_uint16  twobytepw;
 
-    PROCNAME("findFileFormatBuffer");
-
     if (!pformat)
-        return ERROR_INT("&format not defined", procName, 1);
+        return ERROR_INT("&format not defined", __func__, 1);
     *pformat = IFF_UNKNOWN;
     if (!buf)
-        return ERROR_INT("byte buffer not defined", procName, 0);
+        return ERROR_INT("byte buffer not defined", __func__, 0);
 
         /* Check the bmp and tiff 2-byte header ids */
     ((char *)(&twobytepw))[0] = buf[0];
@@ -797,10 +785,8 @@ fileFormatIsTiff(FILE  *fp)
 {
 l_int32  format;
 
-    PROCNAME("fileFormatIsTiff");
-
     if (!fp)
-        return ERROR_INT("stream not defined", procName, 0);
+        return ERROR_INT("stream not defined", __func__, 0);
 
     findFileFormatStream(fp, &format);
     if (format == IFF_TIFF || format == IFF_TIFF_PACKBITS ||
@@ -827,7 +813,7 @@ l_int32  format;
  * Notes:
  *      (1) This is a variation of pixReadStream(), where the data is read
  *          from a memory buffer rather than a file.
- *      (2) On windows, this only reads tiff formatted files directly from
+ *      (2) On Windows, this only reads tiff formatted files directly from
  *          memory.  For other formats, it writes to a temp file and
  *          decompresses from file.
  *      (3) findFileFormatBuffer() requires up to 12 bytes to decide on
@@ -844,12 +830,10 @@ l_int32   format, valid;
 PIX      *pix;
 PIXCMAP  *cmap;
 
-    PROCNAME("pixReadMem");
-
     if (!data)
-        return (PIX *)ERROR_PTR("data not defined", procName, NULL);
+        return (PIX *)ERROR_PTR("data not defined", __func__, NULL);
     if (size < 12)
-        return (PIX *)ERROR_PTR("size < 12", procName, NULL);
+        return (PIX *)ERROR_PTR("size < 12", __func__, NULL);
     pix = NULL;
 
     findFileFormatBuffer(data, &format);
@@ -857,17 +841,17 @@ PIXCMAP  *cmap;
     {
     case IFF_BMP:
         if ((pix = pixReadMemBmp(data, size)) == NULL )
-            return (PIX *)ERROR_PTR( "bmp: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR( "bmp: no pix returned", __func__, NULL);
         break;
 
     case IFF_JFIF_JPEG:
         if ((pix = pixReadMemJpeg(data, size, 0, 1, NULL, 0)) == NULL)
-            return (PIX *)ERROR_PTR( "jpeg: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR( "jpeg: no pix returned", __func__, NULL);
         break;
 
     case IFF_PNG:
         if ((pix = pixReadMemPng(data, size)) == NULL)
-            return (PIX *)ERROR_PTR("png: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("png: no pix returned", __func__, NULL);
         break;
 
     case IFF_TIFF:
@@ -879,45 +863,45 @@ PIXCMAP  *cmap;
     case IFF_TIFF_ZIP:
             /* Reading page 0 by default */
         if ((pix = pixReadMemTiff(data, size, 0)) == NULL)
-            return (PIX *)ERROR_PTR("tiff: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("tiff: no pix returned", __func__, NULL);
         break;
 
     case IFF_PNM:
         if ((pix = pixReadMemPnm(data, size)) == NULL)
-            return (PIX *)ERROR_PTR("pnm: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("pnm: no pix returned", __func__, NULL);
         break;
 
     case IFF_GIF:
         if ((pix = pixReadMemGif(data, size)) == NULL)
-            return (PIX *)ERROR_PTR("gif: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("gif: no pix returned", __func__, NULL);
         break;
 
     case IFF_JP2:
         if ((pix = pixReadMemJp2k(data, size, 1, NULL, 0, 0)) == NULL)
-            return (PIX *)ERROR_PTR("jp2k: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("jp2k: no pix returned", __func__, NULL);
         break;
 
     case IFF_WEBP:
         if ((pix = pixReadMemWebP(data, size)) == NULL)
-            return (PIX *)ERROR_PTR("webp: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("webp: no pix returned", __func__, NULL);
         break;
 
     case IFF_PS:
-        L_ERROR("PostScript reading is not supported\n", procName);
+        L_ERROR("PostScript reading is not supported\n", __func__);
         return NULL;
 
     case IFF_LPDF:
-        L_ERROR("Pdf reading is not supported\n", procName);
+        L_ERROR("Pdf reading is not supported\n", __func__);
         return NULL;
 
     case IFF_SPIX:
         if ((pix = pixReadMemSpix(data, size)) == NULL)
-            return (PIX *)ERROR_PTR("spix: no pix returned", procName, NULL);
+            return (PIX *)ERROR_PTR("spix: no pix returned", __func__, NULL);
         break;
 
     case IFF_UNKNOWN:
         return (PIX *)ERROR_PTR("Unknown format: no pix returned",
-                procName, NULL);
+                __func__, NULL);
         break;
     }
 
@@ -929,10 +913,10 @@ PIXCMAP  *cmap;
             format = IFF_TIFF_G4;
         pixSetInputFormat(pix, format);
         if ((cmap = pixGetColormap(pix))) {
-            pixcmapIsValid(cmap, &valid);
+            pixcmapIsValid(cmap, pix, &valid);
             if (!valid) {
                 pixDestroy(&pix);
-                return (PIX *)ERROR_PTR("invalid colormap", procName, NULL);
+                return (PIX *)ERROR_PTR("invalid colormap", __func__, NULL);
             }
         }
         pixSetPadBits(pix, 0);
@@ -980,8 +964,6 @@ l_int32  format, ret, w, h, d, bps, spp, iscmap;
 l_int32  type;  /* not used */
 PIX     *pix;
 
-    PROCNAME("pixReadHeaderMem");
-
     if (pw) *pw = 0;
     if (ph) *ph = 0;
     if (pbps) *pbps = 0;
@@ -990,9 +972,9 @@ PIX     *pix;
     if (pformat) *pformat = 0;
     iscmap = 0;  /* init to false */
     if (!data)
-        return ERROR_INT("data not defined", procName, 1);
+        return ERROR_INT("data not defined", __func__, 1);
     if (size < 12)
-        return ERROR_INT("size < 12", procName, 1);
+        return ERROR_INT("size < 12", __func__, 1);
 
     findFileFormatBuffer(data, &format);
 
@@ -1000,7 +982,7 @@ PIX     *pix;
     {
     case IFF_BMP:  /* cheating: read the pix */
         if ((pix = pixReadMemBmp(data, size)) == NULL)
-            return ERROR_INT( "bmp: pix not read", procName, 1);
+            return ERROR_INT( "bmp: pix not read", __func__, 1);
         pixGetDimensions(pix, &w, &h, &d);
         pixDestroy(&pix);
         bps = (d == 32) ? 8 : d;
@@ -1011,13 +993,13 @@ PIX     *pix;
         ret = readHeaderMemJpeg(data, size, &w, &h, &spp, NULL, NULL);
         bps = 8;
         if (ret)
-            return ERROR_INT( "jpeg: no header info returned", procName, 1);
+            return ERROR_INT( "jpeg: no header info returned", __func__, 1);
         break;
 
     case IFF_PNG:
         ret = readHeaderMemPng(data, size, &w, &h, &bps, &spp, &iscmap);
         if (ret)
-            return ERROR_INT( "png: no header info returned", procName, 1);
+            return ERROR_INT( "png: no header info returned", __func__, 1);
         break;
 
     case IFF_TIFF:
@@ -1032,18 +1014,18 @@ PIX     *pix;
         ret = readHeaderMemTiff(data, size, 0, &w, &h, &bps, &spp,
                                 NULL, &iscmap, &format);
         if (ret)
-            return ERROR_INT( "tiff: no header info returned", procName, 1);
+            return ERROR_INT( "tiff: no header info returned", __func__, 1);
         break;
 
     case IFF_PNM:
         ret = readHeaderMemPnm(data, size, &w, &h, &d, &type, &bps, &spp);
         if (ret)
-            return ERROR_INT( "pnm: no header info returned", procName, 1);
+            return ERROR_INT( "pnm: no header info returned", __func__, 1);
         break;
 
     case IFF_GIF:  /* cheating: read the pix */
         if ((pix = pixReadMemGif(data, size)) == NULL)
-            return ERROR_INT( "gif: pix not read", procName, 1);
+            return ERROR_INT( "gif: pix not read", __func__, 1);
         pixGetDimensions(pix, &w, &h, &d);
         pixDestroy(&pix);
         iscmap = 1;  /* always colormapped; max 256 colors */
@@ -1052,7 +1034,7 @@ PIX     *pix;
         break;
 
     case IFF_JP2:
-        ret = readHeaderMemJp2k(data, size, &w, &h, &bps, &spp);
+        ret = readHeaderMemJp2k(data, size, &w, &h, &bps, &spp, NULL);
         break;
 
     case IFF_WEBP:
@@ -1062,21 +1044,21 @@ PIX     *pix;
 
     case IFF_PS:
         if (pformat) *pformat = format;
-        return ERROR_INT("PostScript reading is not supported\n", procName, 1);
+        return ERROR_INT("PostScript reading is not supported\n", __func__, 1);
 
     case IFF_LPDF:
         if (pformat) *pformat = format;
-        return ERROR_INT("Pdf reading is not supported\n", procName, 1);
+        return ERROR_INT("Pdf reading is not supported\n", __func__, 1);
 
     case IFF_SPIX:
-        ret = sreadHeaderSpix((l_uint32 *)data, &w, &h, &bps,
+        ret = sreadHeaderSpix((l_uint32 *)data, size, &w, &h, &bps,
                                &spp, &iscmap);
         if (ret)
-            return ERROR_INT( "pnm: no header info returned", procName, 1);
+            return ERROR_INT( "pnm: no header info returned", __func__, 1);
         break;
 
     case IFF_UNKNOWN:
-        return ERROR_INT("unknown format; no data returned", procName, 1);
+        return ERROR_INT("unknown format; no data returned", __func__, 1);
         break;
     }
 
@@ -1125,16 +1107,14 @@ FILE     *fpin;
 PIX      *pix, *pixt;
 PIXCMAP  *cmap;
 
-    PROCNAME("writeImageFileInfo");
-
     if (!filename)
-        return ERROR_INT("filename not defined", procName, 1);
+        return ERROR_INT("filename not defined", __func__, 1);
     if (!fpout)
-        return ERROR_INT("stream not defined", procName, 1);
+        return ERROR_INT("stream not defined", __func__, 1);
 
         /* Read the header */
     if (pixReadHeader(filename, &format, &w, &h, &bps, &spp, &iscmap)) {
-        L_ERROR("failure to read header of %s\n", procName, filename);
+        L_ERROR("failure to read header of %s\n", __func__, filename);
         return 1;
     }
     fprintf(fpout, "===============================================\n"
@@ -1146,19 +1126,19 @@ PIXCMAP  *cmap;
 
     findFileFormat(filename, &format);
     if (format == IFF_JP2) {
-        fpin = lept_fopen(filename, "rb");
+        fpin = fopenReadStream(filename);
         fgetJp2kResolution(fpin, &xres, &yres);
-        fclose(fpin);
+        if (fpin) fclose(fpin);
         fprintf(fpout, "  xres = %d, yres = %d\n", xres, yres);
     } else if (format == IFF_PNG) {
-        fpin = lept_fopen(filename, "rb");
+        fpin = fopenReadStream(filename);
         fgetPngResolution(fpin, &xres, &yres);
-        fclose(fpin);
+        if (fpin) fclose(fpin);
         fprintf(fpout, "  xres = %d, yres = %d\n", xres, yres);
         if (iscmap) {
-            fpin = lept_fopen(filename, "rb");
+            fpin = fopenReadStream(filename);
             fgetPngColormapInfo(fpin, &cmap, &transparency);
-            fclose(fpin);
+            if (fpin) fclose(fpin);
             if (transparency)
                 fprintf(fpout, "  colormap has transparency\n");
             else
@@ -1167,9 +1147,9 @@ PIXCMAP  *cmap;
             pixcmapDestroy(&cmap);
         }
     } else if (format == IFF_JFIF_JPEG) {
-        fpin = lept_fopen(filename, "rb");
+        fpin = fopenReadStream(filename);
         fgetJpegResolution(fpin, &xres, &yres);
-        fclose(fpin);
+        if (fpin) fclose(fpin);
         fprintf(fpout, "  xres = %d, yres = %d\n", xres, yres);
     }
 
@@ -1186,7 +1166,7 @@ PIXCMAP  *cmap;
         l_pngSetReadStrip16To8(0);
 
     if ((pix = pixRead(filename)) == NULL) {
-        L_ERROR("failure to read full image of %s\n", procName, filename);
+        L_ERROR("failure to read full image of %s\n", __func__, filename);
         return 1;
     }
 
@@ -1220,9 +1200,9 @@ PIXCMAP  *cmap;
     if (format == IFF_TIFF || format == IFF_TIFF_G4 ||
         format == IFF_TIFF_G3 || format == IFF_TIFF_PACKBITS) {
         fprintf(fpout, "  Tiff header information:\n");
-        fpin = lept_fopen(filename, "rb");
+        fpin = fopenReadStream(filename);
         tiffGetCount(fpin, &npages);
-        lept_fclose(fpin);
+        if (fpin) fclose(fpin);
         if (npages == 1)
             fprintf(fpout, "    One page in file\n");
         else
@@ -1243,7 +1223,7 @@ PIXCMAP  *cmap;
          * result when a white background is visible through the
          * transparency layer. */
     if (pixGetSpp(pix) == 4) {
-        pixt = pixDisplayLayersRGBA(pix, 0xffffff00, 600.0);
+        pixt = pixDisplayLayersRGBA(pix, 0xffffff00, 600);
         pixDisplay(pixt, 100, 100);
         pixDestroy(&pixt);
     }
@@ -1290,14 +1270,12 @@ BOX       *box;
 PIX       *pixs, *pixc, *pix1, *pix2;
 PIXCMAP   *cmap;
 
-    PROCNAME("ioFormatTest");
-
     if (!filename)
-        return ERROR_INT("filename not defined", procName, 1);
+        return ERROR_INT("filename not defined", __func__, 1);
 
         /* Read the input file and limit the size */
     if ((pix1 = pixRead(filename)) == NULL)
-        return ERROR_INT("pix1 not made", procName, 1);
+        return ERROR_INT("pix1 not made", __func__, 1);
     pixGetDimensions(pix1, &w, &h, NULL);
     if (w > 250 && h > 250) {  /* take the central 250 x 250 region */
         box = boxCreate(w / 2 - 125, h / 2 - 125, 250, 250);
@@ -1334,7 +1312,7 @@ PIXCMAP   *cmap;
          * a colormap.  Although we can write/read 2 bpp BMP, nobody
          * else can read them! */
     if (d == 1 || d == 8) {
-        L_INFO("write/read bmp\n", procName);
+        L_INFO("write/read bmp\n", __func__);
         pixWrite(FILE_BMP, pixc, IFF_BMP);
         pix1 = pixRead(FILE_BMP);
         if (!cmap)
@@ -1343,7 +1321,7 @@ PIXCMAP   *cmap;
             pix2 = pixClone(pix1);
         pixEqual(pixc, pix2, &equal);
         if (!equal) {
-            L_INFO("   **** bad bmp image: d = %d ****\n", procName, d);
+            L_INFO("   **** bad bmp image: d = %d ****\n", __func__, d);
             problems = TRUE;
         }
         pixDestroy(&pix1);
@@ -1351,12 +1329,12 @@ PIXCMAP   *cmap;
     }
 
     if (d == 2 || d == 4 || d == 32) {
-        L_INFO("write/read bmp\n", procName);
+        L_INFO("write/read bmp\n", __func__);
         pixWrite(FILE_BMP, pixc, IFF_BMP);
         pix1 = pixRead(FILE_BMP);
         pixEqual(pixc, pix1, &equal);
         if (!equal) {
-            L_INFO("   **** bad bmp image: d = %d ****\n", procName, d);
+            L_INFO("   **** bad bmp image: d = %d ****\n", __func__, d);
             problems = TRUE;
         }
         pixDestroy(&pix1);
@@ -1367,12 +1345,12 @@ PIXCMAP   *cmap;
         /* PNG works for all depths, but here, because we strip
          * 16 --> 8 bpp on reading, we don't test png for 16 bpp. */
     if (d != 16) {
-        L_INFO("write/read png\n", procName);
+        L_INFO("write/read png\n", __func__);
         pixWrite(FILE_PNG, pixc, IFF_PNG);
         pix1 = pixRead(FILE_PNG);
         pixEqual(pixc, pix1, &equal);
         if (!equal) {
-            L_INFO("   **** bad png image: d = %d ****\n", procName, d);
+            L_INFO("   **** bad png image: d = %d ****\n", __func__, d);
             problems = TRUE;
         }
         pixDestroy(&pix1);
@@ -1380,7 +1358,7 @@ PIXCMAP   *cmap;
 #endif  /* HAVE_LIBPNG */
 
         /* ----------------------- TIFF -------------------------- */
-#if HAVE_LIBTIFF
+#if HAVE_LIBTIFF && HAVE_LIBJPEG
         /* TIFF works for 1, 2, 4, 8, 16 and 32 bpp images.
          * Because 8 bpp tiff always writes 256 entry colormaps, the
          * colormap sizes may be different for 8 bpp images with
@@ -1391,37 +1369,37 @@ PIXCMAP   *cmap;
          * of color entries in pixc. */
 
         /* tiff uncompressed works for all pixel depths */
-    L_INFO("write/read uncompressed tiff\n", procName);
+    L_INFO("write/read uncompressed tiff\n", __func__);
     pixWrite(FILE_TIFF, pixc, IFF_TIFF);
     pix1 = pixRead(FILE_TIFF);
     pixEqual(pixc, pix1, &equal);
     if (!equal) {
         L_INFO("   **** bad tiff uncompressed image: d = %d ****\n",
-               procName, d);
+               __func__, d);
         problems = TRUE;
     }
     pixDestroy(&pix1);
 
         /* tiff lzw works for all pixel depths */
-    L_INFO("write/read lzw compressed tiff\n", procName);
+    L_INFO("write/read lzw compressed tiff\n", __func__);
     pixWrite(FILE_LZW, pixc, IFF_TIFF_LZW);
     pix1 = pixRead(FILE_LZW);
     pixEqual(pixc, pix1, &equal);
     if (!equal) {
         L_INFO("   **** bad tiff lzw compressed image: d = %d ****\n",
-               procName, d);
+               __func__, d);
         problems = TRUE;
     }
     pixDestroy(&pix1);
 
         /* tiff adobe deflate (zip) works for all pixel depths */
-    L_INFO("write/read zip compressed tiff\n", procName);
+    L_INFO("write/read zip compressed tiff\n", __func__);
     pixWrite(FILE_ZIP, pixc, IFF_TIFF_ZIP);
     pix1 = pixRead(FILE_ZIP);
     pixEqual(pixc, pix1, &equal);
     if (!equal) {
         L_INFO("   **** bad tiff zip compressed image: d = %d ****\n",
-               procName, d);
+               __func__, d);
         problems = TRUE;
     }
     pixDestroy(&pix1);
@@ -1429,12 +1407,12 @@ PIXCMAP   *cmap;
         /* tiff jpeg encoding works for grayscale and rgb */
     if (d == 8 || d == 32) {
         PIX  *pixc1;
-        L_INFO("write/read jpeg compressed tiff\n", procName);
+        L_INFO("write/read jpeg compressed tiff\n", __func__);
         if (d == 8 && pixGetColormap(pixc)) {
             pixc1 = pixRemoveColormap(pixc, REMOVE_CMAP_BASED_ON_SRC);
             pixWrite(FILE_TIFF_JPEG, pixc1, IFF_TIFF_JPEG);
             if ((pix1 = pixRead(FILE_TIFF_JPEG)) == NULL) {
-                L_INFO(" did not read FILE_TIFF_JPEG\n", procName);
+                L_INFO(" did not read FILE_TIFF_JPEG\n", __func__);
                 problems = TRUE;
             }
             pixDestroy(&pixc1);
@@ -1450,7 +1428,7 @@ PIXCMAP   *cmap;
             }
             if (diff > 8.0) {
                 L_INFO("   **** bad tiff jpeg compressed image: "
-                       "d = %d, diff = %5.2f ****\n", procName, d, diff);
+                       "d = %d, diff = %5.2f ****\n", __func__, d, diff);
                 problems = TRUE;
             }
         }
@@ -1459,48 +1437,48 @@ PIXCMAP   *cmap;
 
         /* tiff g4, g3, rle and packbits work for 1 bpp */
     if (d == 1) {
-        L_INFO("write/read g4 compressed tiff\n", procName);
+        L_INFO("write/read g4 compressed tiff\n", __func__);
         pixWrite(FILE_G4, pixc, IFF_TIFF_G4);
         pix1 = pixRead(FILE_G4);
         pixEqual(pixc, pix1, &equal);
         if (!equal) {
-            L_INFO("   **** bad tiff g4 image ****\n", procName);
+            L_INFO("   **** bad tiff g4 image ****\n", __func__);
             problems = TRUE;
         }
         pixDestroy(&pix1);
 
-        L_INFO("write/read g3 compressed tiff\n", procName);
+        L_INFO("write/read g3 compressed tiff\n", __func__);
         pixWrite(FILE_G3, pixc, IFF_TIFF_G3);
         pix1 = pixRead(FILE_G3);
         pixEqual(pixc, pix1, &equal);
         if (!equal) {
-            L_INFO("   **** bad tiff g3 image ****\n", procName);
+            L_INFO("   **** bad tiff g3 image ****\n", __func__);
             problems = TRUE;
         }
         pixDestroy(&pix1);
 
-        L_INFO("write/read rle compressed tiff\n", procName);
+        L_INFO("write/read rle compressed tiff\n", __func__);
         pixWrite(FILE_RLE, pixc, IFF_TIFF_RLE);
         pix1 = pixRead(FILE_RLE);
         pixEqual(pixc, pix1, &equal);
         if (!equal) {
-            L_INFO("   **** bad tiff rle image: d = %d ****\n", procName, d);
+            L_INFO("   **** bad tiff rle image: d = %d ****\n", __func__, d);
             problems = TRUE;
         }
         pixDestroy(&pix1);
 
-        L_INFO("write/read packbits compressed tiff\n", procName);
+        L_INFO("write/read packbits compressed tiff\n", __func__);
         pixWrite(FILE_PB, pixc, IFF_TIFF_PACKBITS);
         pix1 = pixRead(FILE_PB);
         pixEqual(pixc, pix1, &equal);
         if (!equal) {
             L_INFO("   **** bad tiff packbits image: d = %d ****\n",
-                   procName, d);
+                   __func__, d);
             problems = TRUE;
         }
         pixDestroy(&pix1);
     }
-#endif  /* HAVE_LIBTIFF */
+#endif  /* HAVE_LIBTIFF && HAVE_LIBJPEG */
 
         /* ----------------------- PNM -------------------------- */
 
@@ -1508,7 +1486,7 @@ PIXCMAP   *cmap;
          * pnm doesn't have colormaps, so when we write colormapped
          * pix out as pnm, the colormap is removed.  Thus for the test,
          * we must remove the colormap from pixc before testing.  */
-    L_INFO("write/read pnm\n", procName);
+    L_INFO("write/read pnm\n", __func__);
     pixWrite(FILE_PNM, pixc, IFF_PNM);
     pix1 = pixRead(FILE_PNM);
     if (cmap)
@@ -1517,7 +1495,7 @@ PIXCMAP   *cmap;
         pix2 = pixClone(pixc);
     pixEqual(pix1, pix2, &equal);
     if (!equal) {
-        L_INFO("   **** bad pnm image: d = %d ****\n", procName, d);
+        L_INFO("   **** bad pnm image: d = %d ****\n", __func__, d);
         problems = TRUE;
     }
     pixDestroy(&pix1);
@@ -1530,12 +1508,12 @@ PIXCMAP   *cmap;
         pix1 = pixConvertTo8(pixc, 1);
     else
         pix1 = pixClone(pixc);
-    L_INFO("write/read gif\n", procName);
+    L_INFO("write/read gif\n", __func__);
     pixWrite(FILE_GIF, pix1, IFF_GIF);
     pix2 = pixRead(FILE_GIF);
     pixEqual(pix1, pix2, &equal);
     if (!equal) {
-        L_INFO("   **** bad gif image: d = %d ****\n", procName, d);
+        L_INFO("   **** bad gif image: d = %d ****\n", __func__, d);
         problems = TRUE;
     }
     pixDestroy(&pix1);
@@ -1550,7 +1528,7 @@ PIXCMAP   *cmap;
     else
         pix1 = pixConvertTo8(pixc, 0);
     depth = pixGetDepth(pix1);
-    L_INFO("write/read jpeg\n", procName);
+    L_INFO("write/read jpeg\n", __func__);
     pixWrite(FILE_JPG, pix1, IFF_JFIF_JPEG);
     pix2 = pixRead(FILE_JPG);
     if (depth == 8) {
@@ -1562,7 +1540,7 @@ PIXCMAP   *cmap;
     }
     if (diff > 8.0) {
         L_INFO("   **** bad jpeg image: d = %d, diff = %5.2f ****\n",
-               procName, depth, diff);
+               __func__, depth, diff);
         problems = TRUE;
     }
     pixDestroy(&pix1);
@@ -1577,13 +1555,13 @@ PIXCMAP   *cmap;
     else
         pix1 = pixClone(pixc);
     depth = pixGetDepth(pix1);
-    L_INFO("write/read webp\n", procName);
+    L_INFO("write/read webp\n", __func__);
     pixWrite(FILE_WEBP, pix1, IFF_WEBP);
     pix2 = pixRead(FILE_WEBP);
     pixCompareRGB(pix1, pix2, L_COMPARE_ABS_DIFF, 0, NULL, &diff, NULL, NULL);
     if (diff > 5.0) {
         L_INFO("   **** bad webp image: d = %d, diff = %5.2f ****\n",
-               procName, depth, diff);
+               __func__, depth, diff);
         problems = TRUE;
     }
     pixDestroy(&pix1);
@@ -1598,7 +1576,7 @@ PIXCMAP   *cmap;
     else
         pix1 = pixConvertTo8(pixc, 0);
     depth = pixGetDepth(pix1);
-    L_INFO("write/read jp2k\n", procName);
+    L_INFO("write/read jp2k\n", __func__);
     pixWrite(FILE_JP2K, pix1, IFF_JP2);
     pix2 = pixRead(FILE_JP2K);
     if (depth == 8) {
@@ -1611,7 +1589,7 @@ PIXCMAP   *cmap;
     lept_stderr("diff = %7.3f\n", diff);
     if (diff > 7.0) {
         L_INFO("   **** bad jp2k image: d = %d, diff = %5.2f ****\n",
-               procName, depth, diff);
+               __func__, depth, diff);
         problems = TRUE;
     }
     pixDestroy(&pix1);
@@ -1619,7 +1597,7 @@ PIXCMAP   *cmap;
 #endif  /* HAVE_LIBJP2K */
 
     if (problems == FALSE)
-        L_INFO("All formats read and written OK!\n", procName);
+        L_INFO("All formats read and written OK!\n", __func__);
 
     pixDestroy(&pixc);
     pixDestroy(&pixs);
